export interface IPropertyData {
    classification: "SystemMetaData" | "CallstackOrException" | "CustomerContent" | "PublicNonPersonalData" | "EndUserPseudonymizedInformation";
    purpose: "PerformanceAndHealth" | "FeatureInsight" | "BusinessInsight";
    comment: string;
    expiration?: string;
    endpoint?: string;
}
export interface IGDPRProperty {
    owner: string;
    comment: string;
    expiration?: string;
    readonly [name: string]: IPropertyData | undefined | IGDPRProperty | string;
}
type IGDPRPropertyWithoutMetadata = Omit<IGDPRProperty, "owner" | "comment" | "expiration">;
export type OmitMetadata<T> = Omit<T, "owner" | "comment" | "expiration">;
export type ClassifiedEvent<T extends IGDPRPropertyWithoutMetadata> = {
    [k in keyof T]: any;
};
export type StrictPropertyChecker<TEvent, TClassification, TError> = keyof TEvent extends keyof OmitMetadata<TClassification> ? keyof OmitMetadata<TClassification> extends keyof TEvent ? TEvent : TError : TError;
export type StrictPropertyCheckError = {
    error: "Type of classified event does not match event properties";
};
export type StrictPropertyCheck<T extends IGDPRProperty, E> = StrictPropertyChecker<E, ClassifiedEvent<OmitMetadata<T>>, StrictPropertyCheckError>;
export {};
