import { GroupIdentifier, IEditorIdentifier, IEditorCloseEvent, IEditorPartOptions, IEditorPartOptionsChangeEvent, SideBySideEditor, EditorCloseContext, IEditorPane, IEditorWillOpenEvent, EditorInputWithOptions } from "../../../common/editor.js";
import { EditorInput } from "../../../common/editor/editorInput.js";
import { IEditorGroup, GroupDirection, IMergeGroupOptions, GroupsOrder, GroupsArrangement, IAuxiliaryEditorPart, IEditorPart } from "../../../services/editor/common/editorGroupsService.js";
import { IDisposable } from "../../../../base/common/lifecycle.js";
import { Dimension } from "../../../../base/browser/dom.js";
import { Event } from "../../../../base/common/event.js";
import { IConfigurationChangeEvent } from "../../../../platform/configuration/common/configuration.js";
import { IConfigurationService } from "../../../../platform/configuration/common/configuration.service.js";
import { IThemeService } from "../../../../platform/theme/common/themeService.service.js";
import { ISerializableView } from "@codingame/monaco-vscode-6980eeab-47bb-5a48-8e15-32caf0785565-common/vscode/vs/base/browser/ui/grid/grid";
import { IEditorService } from "../../../services/editor/common/editorService.service.js";
import { IEditorOptions } from "../../../../platform/editor/common/editor.js";
import { IAuxiliaryWindowOpenOptions } from "@codingame/monaco-vscode-89a82baf-8ded-5b2f-b8af-e5fbd72dc5ad-common/vscode/vs/workbench/services/auxiliaryWindow/browser/auxiliaryWindowService";
import { ContextKeyValue, IContextKey, RawContextKey } from "../../../../platform/contextkey/common/contextkey.js";
export interface IEditorPartCreationOptions {
    readonly restorePreviousState: boolean;
}
export declare const DEFAULT_EDITOR_MIN_DIMENSIONS: Dimension;
export declare const DEFAULT_EDITOR_MAX_DIMENSIONS: Dimension;
export declare const DEFAULT_EDITOR_PART_OPTIONS: IEditorPartOptions;
export declare function impactsEditorPartOptions(event: IConfigurationChangeEvent): boolean;
export declare function getEditorPartOptions(configurationService: IConfigurationService, themeService: IThemeService): IEditorPartOptions;
export interface IEditorPartsView {
    readonly mainPart: IEditorGroupsView;
    registerPart(part: IEditorPart): IDisposable;
    readonly activeGroup: IEditorGroupView;
    readonly groups: IEditorGroupView[];
    getGroup(identifier: GroupIdentifier): IEditorGroupView | undefined;
    getGroups(order?: GroupsOrder): IEditorGroupView[];
    readonly count: number;
    createAuxiliaryEditorPart(options?: IAuxiliaryWindowOpenOptions): Promise<IAuxiliaryEditorPart>;
    bind<T extends ContextKeyValue>(contextKey: RawContextKey<T>, group: IEditorGroupView): IContextKey<T>;
}
export interface IEditorGroupsView {
    readonly windowId: number;
    readonly groups: IEditorGroupView[];
    readonly activeGroup: IEditorGroupView;
    readonly partOptions: IEditorPartOptions;
    readonly onDidChangeEditorPartOptions: Event<IEditorPartOptionsChangeEvent>;
    readonly onDidVisibilityChange: Event<boolean>;
    getGroup(identifier: GroupIdentifier): IEditorGroupView | undefined;
    getGroups(order: GroupsOrder): IEditorGroupView[];
    activateGroup(identifier: IEditorGroupView | GroupIdentifier, preserveWindowOrder?: boolean): IEditorGroupView;
    restoreGroup(identifier: IEditorGroupView | GroupIdentifier): IEditorGroupView;
    addGroup(location: IEditorGroupView | GroupIdentifier, direction: GroupDirection, groupToCopy?: IEditorGroupView): IEditorGroupView;
    mergeGroup(group: IEditorGroupView | GroupIdentifier, target: IEditorGroupView | GroupIdentifier, options?: IMergeGroupOptions): boolean;
    moveGroup(group: IEditorGroupView | GroupIdentifier, location: IEditorGroupView | GroupIdentifier, direction: GroupDirection): IEditorGroupView;
    copyGroup(group: IEditorGroupView | GroupIdentifier, location: IEditorGroupView | GroupIdentifier, direction: GroupDirection): IEditorGroupView;
    removeGroup(group: IEditorGroupView | GroupIdentifier, preserveFocus?: boolean): void;
    arrangeGroups(arrangement: GroupsArrangement, target?: IEditorGroupView | GroupIdentifier): void;
    toggleMaximizeGroup(group?: IEditorGroupView | GroupIdentifier): void;
    toggleExpandGroup(group?: IEditorGroupView | GroupIdentifier): void;
}
export interface IEditorGroupTitleHeight {
    readonly total: number;
    readonly offset: number;
}
export interface IEditorGroupViewOptions {
    readonly preserveFocus?: boolean;
}
export interface IEditorGroupView extends IDisposable, ISerializableView, IEditorGroup {
    readonly onDidFocus: Event<void>;
    readonly onWillOpenEditor: Event<IEditorWillOpenEvent>;
    readonly onDidOpenEditorFail: Event<EditorInput>;
    readonly onDidCloseEditor: Event<IEditorCloseEvent>;
    readonly groupsView: IEditorGroupsView;
    readonly whenRestored: Promise<void>;
    readonly titleHeight: IEditorGroupTitleHeight;
    readonly disposed: boolean;
    setActive(isActive: boolean): void;
    notifyIndexChanged(newIndex: number): void;
    notifyLabelChanged(newLabel: string): void;
    openEditor(editor: EditorInput, options?: IEditorOptions, internalOptions?: IInternalEditorOpenOptions): Promise<IEditorPane | undefined>;
    relayout(): void;
}
export declare function fillActiveEditorViewState(group: IEditorGroup, expectedActiveEditor?: EditorInput, presetOptions?: IEditorOptions): IEditorOptions;
export declare function prepareMoveCopyEditors(sourceGroup: IEditorGroup, editors: EditorInput[], preserveFocus?: boolean): EditorInputWithOptions[];
export interface EditorServiceImpl extends IEditorService {
    readonly onDidOpenEditorFail: Event<IEditorIdentifier>;
    readonly onDidMostRecentlyActiveEditorsChange: Event<void>;
}
export interface IInternalEditorTitleControlOptions {
    readonly skipTitleUpdate?: boolean;
}
export interface IInternalEditorOpenOptions extends IInternalEditorTitleControlOptions {
    readonly supportSideBySide?: SideBySideEditor.ANY | SideBySideEditor.BOTH;
    readonly focusTabControl?: boolean;
    readonly preserveWindowOrder?: boolean;
    readonly inactiveSelection?: EditorInput[];
}
export interface IInternalEditorCloseOptions extends IInternalEditorTitleControlOptions {
    readonly fromError?: boolean;
    readonly context?: EditorCloseContext;
}
export interface IInternalMoveCopyOptions extends IInternalEditorOpenOptions {
    readonly keepCopy?: boolean;
}
